#ifndef BIN2ABC_H
#define BIN2ABC_H

#include "compiler.h"

#define MAX_ADDR 65536

static inline bool is_stdio(const char *filename)
{
    return !filename || !filename[0] || (filename[0] == '-' && !filename[1]);
}

static inline uint8_t *set16(uint8_t * data, uint16_t val)
{
    *data++ = val;
    *data++ = val >> 8;
    return data;
}

static inline uint16_t get16(const uint8_t * data)
{
    return data[0] + (data[1] << 8);
}

static inline uint16_t letohs(uint16_t v)
{
#ifdef WORDS_LITTEENDIAN
    return v;
#else
    return (uint16_t) ((v >> 8) | (v << 8));
#endif
}

FILE *open_file(const char *filename, const char *mode);
int close_file(FILE * f);
size_t read_file(void *buf, size_t buflen, const char *filename);

int readihex(const char *filename, uint8_t * data, bool * isset, int *entryp);

extern bool have_relocs;
void read_reloc_file(const char *filename);
void push_reloc(uint16_t reloc);
int pack_relocs(uint8_t ** ptr, int delta, bool adjust);
void apply_relocs(uint8_t *data, uint16_t origin);

int writeabs(FILE * out, uint8_t * data, const bool * isset,
             int origin, int entrypoint);
int writebac80(FILE * out, const uint8_t * data, int len, int origin,
               int entrypoint, bool driver);
int writebac800(FILE * out, const uint8_t * data, int len, int origin,
                int entrypoint, bool driver);

extern const char *program_name;

#endif /* BIN2ABC_H */
